/*
 	Gasteroids v0.1 aplicacin didctica para dispositivos MIDP2.0
 	
    Copyright (C) 2006  Victor Carceler

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package gpl.gasteroids;

import java.util.Random;
import java.util.Vector;

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Graphics;

/**
 * GasteroidsCanvas implementa la clase en la que tiene lugar la accin del juego.
 * 
 * @author Victor Carceler
 *
 */
public class GasteroidsCanvas extends Canvas implements Runnable {
	private final int QUANTUM = 15;	// Constante de tiempo utilizada para la simulacin
	
	private Vector particles;		// Partculas en la pantalla
	private SpaceShip ship;			// Nave controlada por el jugador
	private Thread thread;			// Hebra que ejecuta la simulacin
	
	private int score = 0;			// Marcador de puntos
	private int lifes = 0;			// Marcador de vidas
	
	/**
	 * Constructor que inicializa el juego.
	 * <p>
	 * Bsicamente:
	 * <ul>
	 * 	<li>Crea unas partculas al azar
	 * 	<li>Crea una nueva nave
	 *  <li>Crea una nueva hebra y la pone en marcha
	 * </ul>
	 */
	public GasteroidsCanvas() {
		particles = getParticles(3);
		ship = getShip();
		
		thread = new Thread(this);
		thread.start();
	}

	/**
	 * Dibuja la escena en la pantalla del mvil.
	 * Es responsable de dibujar todos y cada uno de los pxels de la pantalla
	 * 
	 * @param g		El objeto Graphics en el que dibujar
	 */
	protected void paint(Graphics g) {		
		// Pintamos la pantalla de blanco
		g.setColor(255,255,255);
		g.fillRect(0, 0, getWidth(), getHeight());
		
		// Pintamos cada una de las partculas
		g.setStrokeStyle(Graphics.SOLID);
		for(int i=0; i<particles.size(); i++) {
			Particle p = (Particle)particles.elementAt(i);
			g.setColor(p.r, p.g, p.b);
			int r = (int)(p.m*10);
			g.drawArc((int)(p.x-r/2), (int)(p.y-r/2), r, r, 0, 360);
			g.drawArc((int)p.x, (int)p.y, 1, 1, 0, 360);
		}
		
		// Pintamos la nave (y todos sus proyectiles)
		ship.paint(g);
		
		// Actualizamos el marcador
		g.drawString("Score: " + score + " Lifes: " + lifes, 0, 10, (Graphics.BASELINE | Graphics.LEFT));
	}
	
	/**
	 * Mtodo que ejecuta la hebra encargada de actualizar la simulacin.<p>
	 * La simulacin da saltos en el tiempo de valor QUANTUM.<p>
	 * En cada salto:
	 * <ul>
	 * 	<li>Se actualiza las condiciones de las partculas
	 *  <li>Se actualiza la posicin de la nave (y de todos sus proyectiles)
	 *  <li>Se comprueban las colisiones
	 *  <li>Se redibuja la escena
	 * </ul>
	 */
	public void run() {
		// Cada cierto tiempo actualiza las partculas y redibuja la situacin
		while(true){
			updateParticles();
			updateShip();
			checkCollisions();
			repaint();
			try { Thread.sleep(QUANTUM); } catch (Exception e) {}
		}
	}
	
	/**
	 * Actualiza las condiciones de las partculas.<p>
	 * 
	 * <ul>
	 * 	<li>Se calcula la fuerza a la que est sometida cada partcula (en funcin de dnde est y dnde estn las otras partculas)
	 * 	<li>Se calcula la velocidad de la partcula
	 *  <li>Se calcula la posicin de la partcula
	 * </ul>
	 *
	 */
	public void updateParticles() {
		// Calculamos la fuerza a la que est sometida cada partcula
		for(int i=0; i<particles.size(); i++) {
			Particle p1 = (Particle)particles.elementAt(i);
			p1.resetForce();
			for(int j=0; j<particles.size(); j++) {
				if(i!=j) {
					Particle p2 = (Particle)particles.elementAt(j);
					p1.addForce(p2);
				}
			}
		}
		
		// Calculamos la velocidad de cada partcula y su nueva posicin
		for(int i=0; i<particles.size(); i++) {
			Particle p = (Particle)particles.elementAt(i);
			p.updateSpeed(QUANTUM/10);
			p.updatePosition(QUANTUM/10);
		}
	}
	
	/**
	 * Actualiza la situacin de la nave (posicin y velocidad)
	 *
	 */
	public void updateShip() {
		ship.updateSpeed(QUANTUM/10);
		ship.updatePosition(QUANTUM/10);
	}
	
	/**
	 * Retorna un array con n partculas
	 * 
	 * @param	n		Nmero de partculas a devolver
	 * @return	Un {@link Vector} con las partculas creadas
	 */
	public Vector getParticles(int n) {
		Vector p = new Vector();
		Random r = new Random();
		int width = getWidth();
		int height = getHeight();
		for(int i=0; i<n; i++) {
			Particle x = new Particle();
			x.x = r.nextInt(width);
			x.y = r.nextInt(height);
			x.m = r.nextInt(5) + 1;
			x.x_max = width - 10;	x.x_min = 10;
			x.y_max = height - 10;	x.y_min = 10;
			x.r = r.nextInt(200);	x.g = r.nextInt(200);	x.b = r.nextInt(200);
			p.addElement(x);
		}
		return p;
	}

	/**
	 * Crea una nueva nave que est parada en el centro de la pantalla
	 * 
	 * @return	Nueva nave
	 */
	public SpaceShip getShip() {
		SpaceShip s = new SpaceShip();
		s.x = getWidth()/2;
		s.y = getHeight()/2;
		
		s.x_max = getWidth();
		s.y_max = getHeight();
		
		return s;
	}
	
	
	//	Metodo que se llama cuando pulsamos una tecla
    protected void keyPressed(int keyCode) {
	  int impulsor=getKeyCode(UP);
	  int abajo=getKeyCode(DOWN);
	  int izq=getKeyCode(LEFT);
	  int dcha=getKeyCode(RIGHT);
	  int fuego=getKeyCode(FIRE);

	  if (keyCode == impulsor) {ship.engine_time += 1;}
	  else if (keyCode == abajo) { ship.arg += Math.PI;}
	  else if (keyCode == izq  ) { ship.arg -= Math.PI/10;}
	  else if (keyCode == dcha ) { ship.arg += Math.PI/10;}
	  else if (keyCode == fuego) { ship.fire(); }
    }
    
    // Busca colisiones y acta en consecuencia
    private void checkCollisions() {
    	Vector bullets = ship.getBullets();
    	
    	// Comenzamos buscando colisiones entre proyectiles y Gasteroides
    	// Un proyectil que impacta, desaparece. El Gasteroide se fragmenta o desaparece.
    	for(int i=bullets.size()-1; i>=0; i--) {
    		Bullet b = (Bullet)bullets.elementAt(i);
    		for(int j=particles.size()-1; j>=0; j--) {
    			Particle p = (Particle)particles.elementAt(j);
    			// Si colisionan, destruimos el proyectil y el Gasteroide
    			int x_dist = (int)Math.abs(b.x - p.x);
    			int y_dist = (int)Math.abs(b.y - p.y);
    			int r = (int)(p.m) * 5 - 2;
    			if((x_dist <= r ) && (y_dist <= r)) {
    				score += p.m;
    				bullets.removeElementAt(i);
    				particles.removeElementAt(j);
    				if(p.m>1) {
    					Particle pa = new Particle();
    					Particle pb = new Particle();
    					pa.m = p.m - 1;	pa.x = p.x + 10; pa.y = p.y + 50; pa.r = p.r; pa.g = p.g; pa.b = p.b; pa.x_max = p.x_max; pa.x_min = p.x_min; pa.y_max = p.y_max; pa.y_min = p.y_min;
    					pb.m = p.m - 1; pb.x = p.x - 10; pb.y = p.y - 50; pb.r = p.r; pb.g = p.g; pb.b = p.b; pb.x_max = p.x_max; pb.x_min = p.x_min; pb.y_max = p.y_max; pb.y_min = p.y_min;
    					particles.addElement(pa);
    					particles.addElement(pb);
    				}
    			}
    		}
    	}
    	
    	
    }
}
