/*
 	Gasteroids v0.1 aplicacin didctica para dispositivos MIDP2.0
 	
    Copyright (C) 2006  Victor Carceler

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package gpl.gasteroids;

/**
 * Modela una partcula, es decir, un Gasteroide.<p>
 * 
 * Una partcula cuenta con las siguiente propiedades:
 * <ul>
 * 	<li>Color (R,G,B)
 * 	<li>Posicin (x,y)
 * 	<li>Lmites para la posicin. rea definida por el rectngulo: (x_min,y_min) (x_max,y_max)
 * 	<li>Masa (m) para calcular la fuerza de gravedad entre diferentes partculas
 * 	<li>Carga (q) para calcular la fuerza de atraccin/repulsin entre diferentes partculas ACTUALMENTE NO SE UTILIZA
 *  <li>Velocidad (speed_x, speed_y)
 *  <li>Fuerza a la que est sometida por la influencia de otras partculas (force_x, force_y)
 * </ul>
 * 
 * @author Victor Carceler
 */
public class Particle {
	private static int count = 0;				// Contador de instancias creadas
	private int id;								// id nico para la partcula
	
	public int r,g,b;
	
	public double x,y;							// Posicin X,Y
	public int x_min, x_max, y_min, y_max;		// Lmites a la posicin de la partcula
	public double m;							// Masa
	public double q;							// Carga
	public double speed_x=0.0d, speed_y=0.0d;	// Velocidad actual
	public double force_x=0.0d, force_y=0.0d; 	// Fuerza sobre la partcula
	
	/**
	 * Constructor, crea una nueva partcula
	 *
	 */
	public Particle() {
		id = count++;
	}
	
	public String toString() {
		return "P" + id + " speed: " + speed_x + "," + speed_y + " force: " + force_x + "," + force_y + " pos: " + x + "," + y;
	}
	
	/**
	 * Fija a valor 0 las fuerzas a las que est sometida la partcula
	 */
	public void resetForce() {
		force_x = 0.0d; force_y = 0.0d;
	}
	
	/**
	 * Agrega a las fuerzas de la partcula las que se derivan de la influencia de p
	 * 
	 * @param p Partcula
	 */
	public void addForce(Particle p) {
		double dist;
		
		dist = p.x - x;
		if(Math.abs(dist) > 50) {
			force_x += m*p.m/dist;
		}
		else if(dist != 0.0d){
			force_x -= m*p.m/dist;
		}
		
		dist = p.y - y;
		if(Math.abs(dist) > 50) {
			force_y += m*p.m/dist;
		} else if(dist != 0.0d) {
			force_y -= m*p.m/dist;
		}
	}
	
	/**
	 * Actualiza la velocidad de la partcula suponiendo que han pasado t segundos.
	 * 
	 * @param t	Tiempo en segundos
	 */
	public void updateSpeed(double t) {
		speed_x += t*force_x/m;
		speed_y += t*force_y/m;
	}
	
	/**
	 * Actualiza la posicin de la partcula suponiendo que han pasado t segundos.<p>
	 * La partcula no podr moverse fuera del rea especificada por sus propiedades (x_min, y_min) y (x_max, y_max).
	 * Si la partcula choca con un borde, su posicin no variar y la componente velocidad que la empuja
	 * a traspasar el lmite ser puesta a 0.<p>
	 * Si se quisiese hacer que las partculas rebotasen al chochar con un borde, este es el mtodo adecuado.
	 * @param t Tiempo en segundos
	 */
	public void updatePosition(double t) {
		x += speed_x * t;
		y += speed_y * t;
		if(x<=x_min) {x=x_min; speed_x = 0;}
		if(x>=x_max) {x=x_max; speed_x = 0;}
		if(y<=y_min) {y=y_min; speed_y = 0;} 
		if(y>=y_max) {y=y_max; speed_y = 0;}
		//System.out.println(this);
	}
}
